@@ -26,7 +26,7 @@
 #include "gtksourcegutterrenderer.h"
 #include "gtksourcegutterrenderer-private.h"
 #include "gtksourcestylescheme.h"
-#include "gtksourceview-typebuiltins.h"
+#include "gtksourceview-enumtypes.h"
 #include "gtksourceview-i18n.h"
 
 /**
@@ -80,7 +80,7 @@ enum
 	QUERY_TOOLTIP,
 	QUERY_DATA,
 	QUERY_ACTIVATABLE,
-	NUM_SIGNALS
+	N_SIGNALS
 };
 
 struct _GtkSourceGutterRendererPrivate
@@ -105,7 +105,7 @@ struct _GtkSourceGutterRendererPrivate
 	guint visible : 1;
 };
 
-static guint signals[NUM_SIGNALS] = {0,};
+static guint signals[N_SIGNALS];
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GtkSourceGutterRenderer, gtk_source_gutter_renderer, G_TYPE_INITIALLY_UNOWNED)
 
@@ -1059,7 +1059,21 @@ gtk_source_gutter_renderer_query_data (GtkSourceGutterRenderer      *renderer,
 	g_return_if_fail (start != NULL);
 	g_return_if_fail (end != NULL);
 
-	g_signal_emit (renderer, signals[QUERY_DATA], 0, start, end, state);
+
+	/* Signal emission is relatively expensive and this code path is
+	 * frequent enough to optimize the common case where we only have the
+	 * override and no connected handlers.
+	 *
+	 * This is the same trick used by gtk_widget_draw().
+	 */
+	if (G_UNLIKELY (g_signal_has_handler_pending (renderer, signals[QUERY_DATA], 0, FALSE)))
+	{
+		g_signal_emit (renderer, signals[QUERY_DATA], 0, start, end, state);
+	}
+	else if (GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->query_data)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->query_data (renderer, start, end, state);
+	}
 }
 
 /**
@@ -1122,12 +1136,13 @@ gtk_source_gutter_renderer_set_padding (GtkSourceGutterRenderer *renderer,
 /**
  * gtk_source_gutter_renderer_get_padding:
  * @renderer: a #GtkSourceGutterRenderer
- * @xpad: (out caller-allocates) (allow-none): return location for the x-padding (can be %NULL)
- * @ypad: (out caller-allocates) (allow-none): return location for the y-padding (can be %NULL)
+ * @xpad: (out caller-allocates) (optional): return location for the x-padding,
+ *   or %NULL to ignore.
+ * @ypad: (out caller-allocates) (optional): return location for the y-padding,
+ *   or %NULL to ignore.
  *
  * Get the x-padding and y-padding of the gutter renderer.
- *
- **/
+ */
 void
 gtk_source_gutter_renderer_get_padding (GtkSourceGutterRenderer *renderer,
                                         gint                    *xpad,
@@ -1182,12 +1197,13 @@ gtk_source_gutter_renderer_set_alignment (GtkSourceGutterRenderer *renderer,
 /**
  * gtk_source_gutter_renderer_get_alignment:
  * @renderer: a #GtkSourceGutterRenderer
- * @xalign: (out caller-allocates) (allow-none): return location for the x-alignment (can be %NULL)
- * @yalign: (out caller-allocates) (allow-none): return location for the y-alignment (can be %NULL)
+ * @xalign: (out caller-allocates) (optional): return location for the x-alignment,
+ *   or %NULL to ignore.
+ * @yalign: (out caller-allocates) (optional): return location for the y-alignment,
+ *   or %NULL to ignore.
  *
  * Get the x-alignment and y-alignment of the gutter renderer.
- *
- **/
+ */
 void
 gtk_source_gutter_renderer_get_alignment (GtkSourceGutterRenderer *renderer,
                                           gfloat                  *xalign,
@@ -1314,7 +1330,7 @@ gtk_source_gutter_renderer_set_size (GtkSourceGutterRenderer *renderer,
 /**
  * gtk_source_gutter_renderer_get_background:
  * @renderer: a #GtkSourceGutterRenderer
- * @color: (out caller-allocates) (allow-none): return value for a #GdkRGBA
+ * @color: (out caller-allocates) (optional): return value for a #GdkRGBA
  *
  * Get the background color of the renderer.
  *
@@ -1338,7 +1354,7 @@ gtk_source_gutter_renderer_get_background (GtkSourceGutterRenderer *renderer,
 /**
  * gtk_source_gutter_renderer_set_background:
  * @renderer: a #GtkSourceGutterRenderer
- * @color: (allow-none): a #GdkRGBA or %NULL
+ * @color: (nullable): a #GdkRGBA or %NULL
  *
  * Set the background color of the renderer. If @color is set to %NULL, the
  * renderer will not have a background color.
