@@ -19,8 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GTK_SOURCE_LANGUAGE_PRIVATE_H__
-#define __GTK_SOURCE_LANGUAGE_PRIVATE_H__
+#ifndef GTK_SOURCE_LANGUAGE_PRIVATE_H
+#define GTK_SOURCE_LANGUAGE_PRIVATE_H
 
 #include <glib.h>
 #include "gtksourcecontextengine.h"
@@ -104,5 +104,5 @@ void			  _gtk_source_style_info_free			(GtkSourceStyleInfo       *info);
 
 G_END_DECLS
 
-#endif  /* __GTK_SOURCE_LANGUAGE_PRIVATE_H__ */
+#endif  /* GTK_SOURCE_LANGUAGE_PRIVATE_H */
 
